from transformers import T5Tokenizer, T5ForConditionalGeneration
from collections import Counter
from .prompts import create_input_with_prompt


class MengziZeroShot(object):
    def __init__(self):
        pass

    # def prompt_map(self, task_type, input_string, label_list=None):
    def load(self, model_name="Langboat/mengzi-t5-base-mt"):
        # load model
        self.model_name = model_name
        self.tokenizer = T5Tokenizer.from_pretrained(self.model_name)
        self.model = T5ForConditionalGeneration.from_pretrained(
            self.model_name)

    def token_decode(self, s):
        return self.tokenizer.decode(s, skip_special_tokens=True)

    def pick_most_common(self, x: list) -> str:
        return Counter(x).most_common(1)[0][0]

    def inference(self, task_type, input_string, input_string2=None, entity1=None, entity2=None):
        # make input text
        input_text = create_input_with_prompt(
            task_type, input_string, input_string2, entity1, entity2)
        # tokenize
        encodings = self.tokenizer(
            input_text, max_length=512, padding=True, return_tensors="pt")
        # print('encodings', encodings)

        # model inference
        outputs = self.model.generate(
            encodings['input_ids'], attention_mask=encodings['attention_mask'], max_length=512, num_beams=1)
        # print('outputs', outputs)

        # decode model output
        dec_out = list(map(self.token_decode, outputs))

        # print("dec_out: ", dec_out)
        # return result to web
        return self.pick_most_common(dec_out)



# make inputs with prompt

def task_type_map(task_type):
    task_map = {
        'sentiment_classifier': sentiment_cls,
        'news_classifier': news_cls,
        "medical_domain_intent_classifier": domain_cls,
        "entity_extraction": entity_extr,
        'text_similarity': text_sim,
        'financial_relationship_extraction': finance_extr,
        'ad_generation': ad_gen,
        'comment_object_extraction': com_obj_extr,
    }

    return task_map[task_type]


def create_input_with_prompt(task_type, input_string, input_string2=None, entity1=None, entity2=None):
    prompt_map = task_type_map(task_type)

    if task_type == 'text_similarity':
        return prompt_map(input_string, input_string2)
    elif task_type == 'financial_relationship_extraction':
        return prompt_map(input_string, entity1, entity2)
    return prompt_map(input_string)


def entity_extr(s,):
    '''
    dataset: CLUENER
    task: 实体抽取
    output:
    '''
    prompts = [f'“{s}”找出上述句子中的实体和他们对应的类别']
    return prompts


def text_sim(s1, s2):
    '''
    dataset:
    task: 语义相似度
    output:
    '''
    prompts = [f'“{s1}”和“{s2}”这两句话是在说同一件事吗?']
    return prompts


def finance_extr(s, e1, e2):
    '''
    dataset:
    task: 金融关系抽取
    output:
    '''
    prompts = [f'“{s}”中的“{e1}”和“{e2}”是什么关系？答:']
    return prompts


def ad_gen(s):
    '''
    dataset:
    task: 广告文案生成
    output:
    '''
    prompts = [f'请根据以下产品信息设计广告文案。商品信息:{s}']
    return prompts


def domain_cls(s):
    '''
    dataset:
    task: 医学领域意图分类
    output:
    '''
    # dataset: quake-qic
    prompts = [
        f'问题:“{s}”。此问题的医学意图是什么？选项：病情诊断，病因分析，治疗方案，就医建议，指标解读，疾病描述，后果表述，注意事项，功效作用，医疗费用。']
    return prompts


def sentiment_cls(s):
    '''
    dataset: eprstmt
    task: 评论情感分类
    output: 消极/积极
    '''
    prompts = [f'评论:{s}。请判断该条评论所属类别(积极或消极)并填至空格处。回答：']
    #    f'"{s}"。 如果这个评论的作者是客观的，那么请问这个评论的内容是什么态度的回答？答：',
    #    f'现有机器人能判断句子是消极评论还是积极评论。已知句子：“{s}”。这个机器人将给出的答案是：'
    return prompts


def com_obj_extr(s):
    '''
    dataset:
    task: 评论对象抽取
    output:
    '''
    prompts = [f'评论:{s}.这条评论的评价对象是谁？']
    return prompts


def news_cls(s):
    '''
    dataset: tnews
    task: 新闻分类
    output:
    '''
    label_list = ['故事', '文化', '娱乐', '体育', '财经', '房产', '汽车',
                  '教育', '科技', '军事', '旅游', '国际', '股票', '农业', '电竞']

    prompts = [f'“{s}”是什么新闻频道写的？选项：{"，".join(label_list)}。答：', ]
    #    f'这条新闻是关于什么主题的？新闻：{s}。选项：{"，".join(label_list)}。答：',
    #    f'这是关于“{"，".join(label_list)}”中哪个选项的文章？文章：{s}。 答：']
    return prompts



# -*- coding=utf-8 -*-
"""
# library: mengzi-zero-shot
# author: huajingyun
# license: Apache License 2.0
# email: huajingyun@chuagnxin.com
# github: https://github.com/Langboat/Mengzi
# description: NLU & NLG (zero-shot) depend on mengzi-t5-base-mt pretrained model
# website: www.langboat.com
"""

from .inference import MengziZeroShot

__all__ = ['MengziZeroShot']
__version__ = '0.1.0'



from distutils.core import setup
import os

with open("README.rst", "r") as f:
    long_description = f.read()

DIR_PATH = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(DIR_PATH, 'requirements.txt'),
          'r', encoding='utf-8') as f:
    requirements = f.readlines()

setup(name='mengzi-zero-shot',  # 包名
      version='0.1.1',  # 版本号
      description='NLU & NLG (zero-shot) depend on mengzi-t5-base-mt',
      long_description=long_description,
      author='huajingyun',
      author_email='huajingyun@hotmail.com',
      url='https://github.com/Langboat/mengzi-zero-shot',
      install_requires=requirements,
      license='Apache License 2.0',
      packages=['mengzi_zs'],
      platforms=["all"],
      classifiers=[
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python :: 3.7',
          'Topic :: Scientific/Engineering :: Artificial Intelligence',
      ],
      keywords='machine-learning,zero-shot,NLU,NLG,NLP',
      )
